import cocos
from cocos.actions import *

from cbl import scene
from cbl import actors
from cbl import back_to_apartment

class Scene(scene.SceneLayer):
    map_file = 'apartment_exterior.svg'

    def open_scene(self):
        # add the player
        self.actors['player'] = actors.Player()
        self.action_layer.add(self.actors['player'])
        self.add_actor('partner', actors.Partner, 'target_partner_start')

        self.actors['partner'].sprite.do(
            MoveBy((100, 0), duration=2) +
            CallFunc(self.check_distance)
        )
        self.triggers_disabled['trigger_door'] = True

    def check_distance(self):
        if 'trigger_shooters' in self.triggers_disabled:
            return
        d = self.actors['partner'].sprite.x - self.actors['player'].sprite.x
        if d > 100:
            self.say('partner', "Are you coming?")

    def shoot(self, actor):
        if actor.shots:
            target, hit = actor.shots.pop(0)
            self.add_bullet(actor.sprite.get_rect(),
                -1,
                self.actors[target].sprite.get_rect(),
                hit)
            if hit:
                if target == 'player':
                    self.you_died()
                else:
                    self.actors['partner'].died()
                    print 'DISABLE', self.triggers_disabled
                    del self.triggers_disabled['trigger_door']

    def trigger_door(self):
        self.enable_action('Run away', self.leave)
        self.triggers_disabled['trigger_door'] = True

    def leave(self):
        self.new_scene(back_to_apartment.Scene())
        self.disable_action()

    def trigger_shooters(self):
        self.triggers_disabled['trigger_shooters'] = True
        self.add_actor('npc1', actors.ShooterNPC, 'target_npc1',
            shots=[('partner', False), ('partner', False),
                ('player', False), ('player', False), ('player', True)])
        self.add_actor('npc2', actors.ShooterNPC, 'target_npc2',
            shots=[('partner', False), ('partner', True),
                ('player', False), ('player', False), ('player', True)])

        self.actors['npc1'].sprite.do(
            MoveBy((-100, 0), duration=2) + 
            Repeat(CallFunc(self.shoot, self.actors['npc1']) +
                RandomDelay(1, 2))
        )

        self.actors['npc2'].sprite.do(
            MoveBy((-100, 0), duration=2) + 
            Repeat(CallFunc(self.shoot, self.actors['npc2']) +
                RandomDelay(0.5, 1.5))
        )

