
import cocos
from cocos.actions import *

from cbl import scene
from cbl import actors

from cbl import apt_ext_lost
from cbl import apt_ext_shooting
from cbl import apt_ext_chase

class Scene(scene.SceneLayer):
    map_file = 'back_to_apartment.svg'

    def open_scene(self):
        # add the player
        self.actors['player'] = actors.Player()
        self.action_layer.add(self.actors['player'])
        self.do(Delay(5) + CallFunc(self.do_baddie))

    def trigger_hide(self):
        self.enable_action('Hide', self.do_hide)
    def untrigger_hide(self):
        self.disable_action()

    def do_hide(self):
        self.actors['player'].do_hide()
        self.disable_action()

    def npc_gone(self):
        self.enable_action('Leave hiding', self.do_unhide)
        self.remove_blocker('door_blocker')

    def do_unhide(self):
        self.actors['player'].do_unhide()
        self.disable_action()

    def do_baddie(self):
        self.do(
            Delay(1) + 
            CallFunc(self.bang_door) +
            Delay(.4) + 
            CallFunc(self.bang_door) +
            Delay(1) + 
            CallFunc(self.open_door) +
            CallFunc(self.add_shooter)
        )

    def shoot(self, actor):
        if actor.shots:
            target, hit = actor.shots.pop(0)
            self.add_bullet(actor.sprite.get_rect(),
                -1,
                self.actors[target].sprite.get_rect(),
                hit)
            if hit:
                self.you_died()

    def bang_door(self):
        t = self.triggers.paths_by_label['bang_focus'][0].get_box()
        s = cocos.sprite.Sprite('bang.png', t.center)
        self.gui_layer.add(s)
        s.do(FadeOut(duration=0.2) +
            CallFunc(lambda s=s:s.parent.remove(s)))

    def open_door(self):
        t = self.triggers.paths_by_label['bang_focus'][0].get_box()
        s = cocos.sprite.Sprite('bang.png', t.center)
        self.gui_layer.add(s)
        s.do(FadeOut(duration=0.2) +
            CallFunc(lambda s=s:s.parent.remove(s)))

        t = self.triggers.paths_by_label['door_open_focus'][0].get_box()
        s = cocos.sprite.Sprite('door.png', t.bottomleft, anchor=(0,0))
        self.map_layers['background'].add(s)

    baddie_ran_away = False
    def timeout(self):
        self.baddie_ran_away = True

    def trigger_player_exit(self):
        # player has left the building
        if self.baddie_ran_away:
            self.new_scene(apt_ext_lost.Scene())
        elif self.actors['player'].has_gun:
            self.new_scene(apt_ext_shooting.Scene())
        else:
            self.new_scene(apt_ext_chase.Scene())

    def trigger_gun(self):
        self.enable_action('Get gun', self.get_gun)
    def untrigger_gun(self):
        self.disable_action()

    def get_gun(self):
        self.actors['player'].has_gun = True
        self.disable_action()
        self.triggers_disabled['trigger_gun'] = True
        self.hint('Press LEFT SHIFT to draw your gun and SPACE to shoot')

    def player_shoot(self):
        if 'npc' not in self.actors:
            # XXX shoot at something random
            return

        self.add_bullet(self.actors['player'].sprite.get_rect(),
            self.actors['player'].last_move,
            self.actors['npc'].sprite.get_rect(),
            hit=False)
        print 'BANG'

        if self.actors['npc'].is_runnning:
            return
        self.actors['npc'].is_runnning = True

        # XXX this is easier?
        for a in self.actors['npc'].sprite.actions:
            print 'CANCELLED', a
            a.scheduled_to_remove = True

        x = self.actors['npc'].sprite.x
        self.actors['npc'].sprite.do(
            MoveBy((800-x, 0), duration=1) + 
            CallFunc(self.npc_gone)
        )

    def add_shooter(self):
        npc = self.add_actor('npc', actors.ShooterNPC, 'target_npc',
            shots=[('player', False), ('player', True)],
            is_runnning=False)

        if self.actors['player'].hidden:
            npc.sprite.do(
                MoveBy((-100, 0), duration=2) + 
                Delay(1) +
                MoveBy((-100, 0), duration=2) + 
                Delay(1) +
                MoveBy((-100, 0), duration=2) + 
                Delay(1) +
                MoveBy((400, 0), duration=2) + 
                CallFunc(self.npc_gone)
            )
        else:
            npc.sprite.do(
                MoveBy((-100, 0), duration=2) + 
                Delay(.1) +
                Repeat(CallFunc(self.shoot, npc) +
                    RandomDelay(1, 2))
            )

