
import pyglet
from pyglet.window import key
from pyglet.gl import glPushMatrix, glPopMatrix, glColor4f
import cocos
from cocos.actions import Delay, CallFuncS, CallFunc
from cbl import ninepatch

class SpeechLayer(cocos.layer.ScrollableLayer):
    def __init__(self, text, position, callback, duration=2):
        super(SpeechLayer, self).__init__()
        self.text = text
        self.text_position = position
        self.callback = callback
        self.duration = duration
        image = pyglet.resource.image('speech_bubble.png')
        self.speech_bubble = ninepatch.NinePatch(image)
        self.label = pyglet.text.Label(text, font_size=16, anchor_y='bottom',
            multiline=True, width=200, color=(0 ,0 ,0 ,255))

    def set_batch(self, batch):
        # sprite .remove() wants to call me with None... meh
        pass

    def on_enter(self):
        super(SpeechLayer, self).on_enter()
        actions = Delay(self.duration)
        if self.callback is not None:
            actions = actions + CallFunc(self.callback)
        actions = actions + CallFuncS(self.parent.remove)
        self.do(actions)

    def draw(self):
        # XXX boo, squirtle
        glColor4f(1, 1, 1, 1)

        glPushMatrix()
        self.transform()

        width = self.label.content_width
        height = self.label.content_height
        x, y = self.text_position
        self.label.x = int(x - width / 2)
        self.label.y = int(y + 16)
        self.speech_bubble.draw_around(self.label.x, self.label.y, width, height)
        self.label.draw()
        glPopMatrix()


class ActionLayer(cocos.layer.Layer):
    def __init__(self, text, position):
        super(ActionLayer, self).__init__()
        self.text = text
        self.position = position
        image = pyglet.resource.image('action_bubble.png')
        self.speech_bubble = ninepatch.NinePatch(image)
        self.label = pyglet.text.Label(text, font_size=16, anchor_y='top',
            anchor_x='center', color=(255 ,255 ,255 ,255))

    def set_batch(self, batch):
        # sprite .remove() wants to call me with None... meh
        pass

    def draw(self):
        # XXX boo, squirtle
        glColor4f(1, 1, 1, 1)

        glPushMatrix()
        self.transform()

        width = self.label.content_width
        height = self.label.content_height
        self.speech_bubble.draw_around(-width // 2, -height, width, height)
        self.label.draw()
        glPopMatrix()

